#!/bin/bash
#

PORT_RU=":81"
HELLO_RU="hello from ru"
TEST_USER_RU=wcgiappuser1

PORT_PY=":82"
HELLO_PY="hello from py!"
TEST_USER_PY=rackappuser1

function show_usage
{
    cat << SHOW_HELP
Inits test environment for testing alt-mod-passenger suit working 
with an Apache, selected with <param>.

    $0 <param>

    where <param> is one of:
      --httpd      for apache 2.2. Packet 'alt-mod-passenger'
      --httpd24    for apache 2.4. Packet 'httpd24-alt-mod-passenger'
      --ea4        for EasyApache4. Packet 'ea-apache24-alt-mod-passenger'
      --help       show this help.

Note: The script assumes that selected Apache service is installed and working properly.
SHOW_HELP

exit 1
}

function print_m_dot {
  local STR="$1"
  local MAX_LEN=$2
  local LEN=${#STR}  # len of S

  local L=$[ $MAX_LEN - $LEN ]
  [ $MAX_LEN -le $LEN ] && {  # MAX <= LEN
    L=1
  }

  echo -n "$STR";
  head -c $L < /dev/zero | tr '\0' '.';
}

function check_installed
{
  local NAME="$1"
#  rpm -q $NAME &>/dev/null
  rpm -q $NAME
}

function install_pack
{
  local NAME="$1"
  local INSTALLED=0
  local S=''
  local H="Check installed $NAME "
#  echo -n "$H"
  S=$(check_installed "$NAME") && {
    print_m_dot "$H" 50
    echo " OK: $S"
  } || {
    echo ""
    yum install -y $NAME
    INSTALLED=1

    # check installation again
    check_installed "$NAME" || {
      echo "NOT installed $NAME! EXIT."
      exit 2
    }
  }

  return $INSTALLED
}

BASE_DIR=''
HTTPD=httpd
APACHE=httpd
GRANT_ACCESS="Require all granted"

case "$1" in
   '' )
       show_usage
       ;;
   '--help' )
       show_usage
       ;;
   '--httpd' )
       V=2
       EX=""
       echo "Make test environment for 'alt-mod-passenger'"
       ;;
   '--httpd24' )
       V=4
       BASE_DIR=/opt/rh/httpd24/root
       HTTPD=httpd24-httpd
       EX=httpd24-
       ;;
   '--ea4' )
       V=ea4
       APACHE=apache2
       EX=ea-apache24-
       ;;
esac

# Run as root
if [ "$UID" -ne "0" ]
then
  echo "Must be root to run this script."
  sudo $0 $*
  exit 1
fi

echo "----------- START ---------"

HTTPD_CMD=$BASE_DIR/usr/sbin/httpd

PACK_RPM="${EX}alt-mod-passenger"
echo "Make test environment for '$PACK_RPM'"

# Install CafeFS and init skeleton

install_pack cagefs
# check if skeleton folder exists and not empty to perform init
[ -e /usr/share/cagefs-skeleton ] && ls /usr/share/cagefs-skeleton/* &>/dev/null || {
  /usr/sbin/cagefsctl --init
}

install_pack wget


# install XXX-alt-mod-passenger pack with deps and supporting apps

check_installed $PACK_RPM || {

  # Install tools to build mod_passenger.so
  LIST_APP="gcc-c++ openssl-devel zlib-devel rubygem-rake ruby-devel "

  [ "$V" == "2" ] && {
    LIST_APP="$LIST_APP httpd httpd-devel apr-util-devel apr-devel"
  } || {
    [ "$V" == "4" ] && {
      LIST_APP="$LIST_APP httpd24-httpd httpd24-httpd-devel httpd24-apr-devel httpd24-apr-util-devel apr-devel"
    } || {
      [ "$V" == "ea4" ] && {
        LIST_APP="$LIST_APP ea-apache24 ea-apache24-devel ea-apache24 ea-apr-devel"
      } || {
        show_usage
      }
    }
  }

  echo "Install deps soft to build mod_passenger.so:"
  for PACK in $LIST_APP
  do
    install_pack $PACK
  done

#  echo "Install passenger:"
#  install_pack $PACK_RPM
}

echo "Install soft for tests:"
install_pack alt-python27
INST1=$?

install_pack alt-ruby21
INST2=$?


APACHE_V=`rpm -q --qf %{VERSION} httpd`

cat << EOF
Add/use users:
    For python: $TEST_USER_PY
    For Ruby  : $TEST_USER_RU

EOF

# Create new user for Python app

echo "Enable Python user '$TEST_USER_PY'"
useradd -m $TEST_USER_PY
cagefsctl --enable $TEST_USER_PY


# Create new user for Ruby app

echo "Enable Ruby user '$TEST_USER_RU'"
useradd -m $TEST_USER_RU
cagefsctl --enable $TEST_USER_RU


# Check passenger against CageFS

CAGEFS_MP=/etc/cagefs/cagefs.mp

echo -n "Check $PACK_RPM registered in CageFS ..."
grep -e '/opt$' $CAGEFS_MP &>/dev/null && grep -e '^/var/passenger' $CAGEFS_MP &>/dev/null && {
  echo " OK."
} || {
  echo " FAILED. Check $CAGEFS_MP."
}


# Make environment for users
# For user $TEST_USER_PY

echo "Make Python environment for $TEST_USER_PY"

U=$TEST_USER_PY
UHOME=/home/$U
mkdir -p $UHOME/public
mkdir -p $UHOME/tmp
touch $UHOME/tmp/restart.txt

chown -R $U:$U $UHOME
chmod go+rx $UHOME

cat > "$UHOME/passenger_wsgi.py" <<EOT
def application(environ, start_response):
  start_response('200 OK', [('Content-Type', 'text/plain')])
  return [b"${HELLO_PY}\n"]
EOT

mkdir -p $BASE_DIR/etc/$APACHE/conf.d

cat > "$BASE_DIR/etc/$APACHE/conf.d/wcgi_example.conf" <<CONF
# Deploying a Ruby on Rails application: an example

# Suppose you have a Rails application in /somewhere. Add a virtual host to
# your Apache configuration file and set its DocumentRoot to /somewhere/public:
#
Listen *${PORT_PY}
<VirtualHost *${PORT_PY}>
      PassengerAppEnv development
      ServerName www.wcgi-example.com
      PassengerPython /opt/alt/python27/bin/python

  # be sure to point to 'public'!
      DocumentRoot $UHOME/public

      PassengerUser $U
      PassengerGroup $U

      <Directory $UHOME/public>
         # relax Apache security settings
         AllowOverride all
        # MultiViews must be turned off
         Options -MultiViews
         ${GRANT_ACCESS}
      </Directory>
</VirtualHost>
CONF



# Make environment for users
# For user $TEST_USER_RU

echo "Make Ruby environment for $TEST_USER_RU"

U=$TEST_USER_RU
UHOME=/home/$U

mkdir -p $UHOME/public
mkdir -p $UHOME/tmp
touch $UHOME/tmp/restart.txt

chown -R $U:$U $UHOME
chmod go+rx $UHOME

cat > "$UHOME/config.ru" <<EOT
app = proc do |env|
    [200, { "Content-Type" => "text/html" }, ["${HELLO_RU}\n"]]
end
run app
EOT

mkdir -p /etc/$APACHE/conf.d

cat > "$BASE_DIR/etc/$APACHE/conf.d/rack_example.conf" <<CONF
# Deploying a Ruby on Rails application: an example

# Suppose you have a Rails application in /somewhere. Add a virtual host to
# your Apache configuration file and set its DocumentRoot to /somewhere/public:
#
Listen *${PORT_RU}
<VirtualHost *${PORT_RU}>
      ServerName www.rack-example.com
      PassengerRuby /opt/alt/ruby21/bin/ruby

  # be sure to point to 'public'!
      DocumentRoot $UHOME/public

      PassengerUser $U
      PassengerGroup $U

      <Directory $UHOME/public>
         # relax Apache security settings
         AllowOverride all
        # MultiViews must be turned off
         Options -MultiViews
         ${GRANT_ACCESS}
      </Directory>
</VirtualHost>
CONF


# Make environment for apache

echo "Make app environment for $TEST_USER_RU"

(( $INST1 || $INST2 )) && {
  echo "Update cagefs cache..."
  cagefsctl --force
}

echo "Restart HTTPD: $HTTPD"
service $HTTPD restart

echo -n "Check passenger is running..."
$HTTPD_CMD -M | grep passenger &>/dev/null && {
  echo " OK"
} || {
  echo " FAILED"
}

echo
echo "Test Environment..."

echo
print_m_dot " .) test python: " 50
OUT_PY=$(wget http://localhost${PORT_PY}/ -O- -q)
[ "$OUT_PY" == "$HELLO_PY" ] && { echo "  OK!"; } || { echo "  FAILED!!!"; }
echo $OUT_PY

echo
print_m_dot " .) test ruby: " 50
OUT_RU=$(wget http://localhost${PORT_RU}/ -O- -q)
[ "$OUT_RU" == "$HELLO_RU" ] && { echo "  OK!"; } || { echo "  FAILED!!!"; }
echo $OUT_RU

echo "----------- DONE ----------"

